
"""
 * 过滤器类
 *<B>说明：</B>
 *<pre>
 *  模板的调用的方法必须注入到此对象
 *</pre>
"""
class Filters:

    def __init__(self):

        self.template = None

        return ;


    def setTemplate(self,template):

        self.template = template

        return ;

